/*	RasTherm watchdog
	v1, March 2014

	By bertrand Zauhar, VE2ZAZ
	http://ve2zaz.net

	TARGET device: PIC12F683
	Compiler: SourceBoost's BoostC for PIC
	IDE: SourceBoost IDE

*/

#include <PIC12F683.h>  // Probably redundant
#include <system.h>
#include <string.h>
// #include <icd2.h>   // Only needed if real time debugging using ICD2 is required

// GLOBAL VARIABLES
unsigned char 			delay_min_ctr 	=	0;
unsigned char 			delay_sec_ctr 	=	0;
unsigned char			no_pulse_sec_ctr=	0;		
bool					must_reset 		=	false;
bool					old_wd_pulse 	=	false;
bool					new_wd_pulse 	= 	false;
bool					bit1_read		=	false;
bool					bit2_read		=	false;
bool					pulse_detected	=	false;

// CONFIGURATION BYTES
#pragma DATA _CONFIG, _FCMEN_OFF & _IESO_OFF & _BOD_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_OFF & _WDT_OFF & _INTOSCIO
#pragma CLOCK_FREQ 125000		// Tells the compiler what clock frequency is used. Important for delay functions

// CONSTANTS AND MASKS DEFINITION

#define wd_heat				0 	//The bit position of the GPIO port corresponding to the heat relay sense signal.
#define wd_cool				1 	//The bit position of the GPIO port corresponding to the cool relay sense signal.
#define wd_pulse			2	//The bit position of the GPIO port corresponding to the Pi watchdow pulse sense signal.
#define pi_reset_not		5	//the bit position of the GPIO port corresponding to the Pi Reset_Not signal.
#define latched_reset		4	//the bit position of the GPIO port corresponding to the Latched reset sitgnal.


// Interrupt Service Routine
/* 
void interrupt( void )
{
}	
*/
// End of interrupt routine


// EXECUTED CODE STARTS HERE
void main(void)
{
	// Initialization stage
	intcon 		= 	0b00000000; 	// disable interrupts.	
	ioc    		=	0b00000000;		// disable gpio interrupts on change
	pie1   		=	0b00000000; 	// disable various interrupts
	option_reg 	= 	0b00001111;		// weak pullups global enable, prescaler assigned to WDT and set to 1:128, not used
	pcon		=	0b00000000;		// Low power wake up and brown out reset disabled
	wpu    		=	0b00110111;		// individual weak pullups enabled
	osccon		=	0b00010001; 	// select internal oscillator and set it to 31Khz	
	t1con		=	0b00110000;		// disable timer1 and set prescaler div. by 8 
	t2con		=	0b01111010;		// disable timer2 and set prescaler div. by 16 and postscaler div. by 16 
	adcon0		=	0b00000000;		// disable A/D converter
	ansel		=	0b00000000;		// Enable analog assignment to pin CIN- (GP1)
	cmcon0		=	0b00000111;		// disable comparator
	cmcon1 		=	0b00000000;		// comparator gating timer 1 asynchronously
	vrcon 		=	0b00000000;		// Disable internal comparator  
	wdtcon		=	0b00000000;		// Assign the right prescaler value to the watchdog timer. Keep WTD off for now.
	gpio 		=	0b00100000;		// Make Reset_Not pin high before assigning as output. Lows on other Port pins.
	trisio 		=	0b00001111;		// GPIO first disabled, but assigned as follows:    
									  //	5-pin2 Reset_Not output connected to Pi's reset pin
									  //	4-pin3 Latched output signalling that a at least one ZAZtherm reset has occured.
									  //	3-pin4 Input Only. Currently not used. Assigned to MCLR input. External pull required when used as GPI
									  //	2-pin5 watchdog pulse input (sent by Pi)
									  //	1-pin6 cool relay sense input
									  //	0-pin7 heat relay sense input
	delay_ms(250);					// Generates a startup delay of 0.5 second
	delay_ms(238);					// 

	// Actual thermostat analysis code starts here
	clear_bit(gpio,latched_reset); // clear the latched reset pin
    delay_s(120); 	// Give enough time for Pi to boot up initially
	while(true)   // passes here every second or so.
	{	
		// Here we check for pulse presence coming from Pi 
		pulse_detected = false;		// initialize pulse_detected variable
		old_wd_pulse = test_bit(gpio,wd_pulse); // Take a firse pulse line reading
		delay_ms(250);					// wait 0.5 s (adjusted)
		delay_ms(238);					// 
		new_wd_pulse = test_bit(gpio,wd_pulse);  // Take another reading
		if (new_wd_pulse != old_wd_pulse) pulse_detected = true; // If the two readings different, set the detection flag
		delay_ms(250);					// wait 0.5 s (adjusted)
		delay_ms(238);					// 
		new_wd_pulse = test_bit(gpio,wd_pulse);  // Take another reading
		if (new_wd_pulse != old_wd_pulse) pulse_detected = true; // If the two readings different, set the detection flag
		if (!pulse_detected)  			// If still no pulse detected try one last time
		{	
			delay_ms(50);					// wait 0.05 s
			new_wd_pulse = test_bit(gpio,wd_pulse);  // Take another new reading
			if (new_wd_pulse != old_wd_pulse) pulse_detected = true; // If the two readings different, set the detection flag
		}
		no_pulse_sec_ctr = no_pulse_sec_ctr + 1;  // Increment the missing pulse seconds counter
		if (pulse_detected) no_pulse_sec_ctr = 0;   //  If a pulse was detected, reset the missing pulse seconds counter
		if (no_pulse_sec_ctr >= 120) must_reset = true;   // If no pulse detected in 120 seconds, must reset Pi.
		delay_sec_ctr = delay_sec_ctr + 1;	// increments by one second 
		if (delay_sec_ctr == 60) 	// Minutes counter rollover
		{
			delay_sec_ctr = 0;						// Reset seconds counter
			delay_min_ctr = delay_min_ctr + 1; 		// increments by one minute
		}
		bit1_read = test_bit(gpio,wd_heat);  // Check whether heat relay is active
		if (bit1_read) 	// if heating relay not active, reset minute delay counter
		{
			delay_sec_ctr = 0;			// Reset seconds counter
			delay_min_ctr = 0;			// Reset minutes counter
		}
		if (delay_min_ctr >= 120) must_reset = true;  // Pi thermostat has been heating for two hours, must reset Pi.
		// checking for both heat and cool relays activated simultaneously, which is a fault
		bit1_read = test_bit(gpio,wd_heat);		//Read heat relay state
		bit2_read = test_bit(gpio,wd_cool); 	//Read cool relay state
		if ((!bit1_read) && (!bit2_read)) must_reset = true; // both relays active, must reset
		if (must_reset)   // One of the test conditions failed (must reset), reset Pi now.
		{
			clear_bit(gpio,pi_reset_not); // Activate reset line low
			delay_ms(250);					// Generates a delay of 0.5 second
			delay_ms(250);					// "
			set_bit(gpio,pi_reset_not); // release reset line high
			must_reset = false;
			set_bit(gpio,latched_reset); // set the latched reset pin high
			delay_sec_ctr = 0;		// Clear the various counters
			delay_min_ctr = 0;		// "
			no_pulse_sec_ctr = 0;	// "
			delay_s(120); 	// give enough time for Pi to boot up
		}
	}
}

